#ifndef HUMANDETECTIONMANAGE_H
#define HUMANDETECTIONMANAGE_H
#include "HumanDetection.h"
#include "Common.h"
#include "VidesData.h"
#include "ScopeSemaphoreExit.h"
#include "LicensePlateRecognition.h"
#include "FaceReconitionHandle.h"
#include <QThread>
#include <random>
#include <QSemaphore>
#include <vector>
#include <opencv2/opencv.hpp>
#include <mutex>
class AlgorithmTaskManage{
public:
    AlgorithmTaskManage(int humanDetectionLen,int licensePlateLen,int faceLen);
    ~AlgorithmTaskManage();
    static AlgorithmTaskManage& getInstance(int humanDetectionLen,int licensePlateLen,int faceLen)
    {
        static AlgorithmTaskManage instance(humanDetectionLen,licensePlateLen,faceLen);
        return instance;
    }
    void initHumanDetectionManage(const QString &modelPaths,
                                  float carShapeConfidence,float &height_reference,int &uniformColor);

    void initLicensePlateManage(const QString &modelPaths,
                                float carConfidence);

    void modifyImageFeature(std::map<QString,QString>&maps,int numberFaces,float confidence,bool isNull);
    void initFaceReconitionHandle(std::map<QString,QString>&maps,int numberFaces,float confidence);
    
    void *schedulingAlgorithm(int scheType);
    
    

    int executeFindHuManCar(const cv::Mat &source,int res,std::vector<vides_data::ParkingArea> &currentPlate,
                            std::map<int,int>&resMap, QString sSn);

    void executeFindlicensePlateNumber(const cv::Mat &source, QString &lpNumber,vides_data::requestLicensePlate &plate,
                                       qint64 currentTime,QString sSn);
    
    void executeFindDoesItExistEmployee(const cv::Mat &source,std::list<vides_data::faceRecognitionResult>&faces,QString sSn);


private:
    template<typename T>
    T* schedulingAlgorithmTemplate(std::vector<T*>& objects, std::mutex& mtx) {
        std::lock_guard<std::mutex> lock(mtx);
        qint64 currentTime = QDateTime::currentMSecsSinceEpoch();
        qint64 maxWaitTime = 0;
        std::vector<T*> schedulableObjects;

        for (T* obj : objects) {
            if (obj->getIsRunning()) continue;
            qint64 waitTime = std::abs(currentTime - obj->getThreadTime());
            if (waitTime > maxWaitTime) {
                schedulableObjects.clear();
                schedulableObjects.push_back(obj);
                maxWaitTime = waitTime;
            } else if (waitTime == maxWaitTime) {
                schedulableObjects.push_back(obj);
            }
        }
        if (schedulableObjects.empty()) {
            return nullptr;
        }
        if (schedulableObjects.size() == 1) {
            return schedulableObjects.at(0);
        }

        std::random_device rd;
        std::mt19937 gen(rd());
        std::uniform_int_distribution<> dis(0, schedulableObjects.size() - 1);
        return schedulableObjects[dis(gen)];
    }

    static AlgorithmTaskManage* instance;

    std::vector<HumanDetection*>humanDetections;
    
    std::vector<LicensePlateRecognition*>licensePlateRecognitions;

    std::vector<FaceReconitionHandle*>faceReconitionHandles;
    
    int humanDetectionLen;

    int licensePlateLen;

    int faceLen;

    QSemaphore semaphore;
    
    QSemaphore plateSemaphore;

    QSemaphore faceSemaphore;

    std::mutex mtxHuman;

    std::mutex mtxLicense;

    std::mutex mtxFace;


    

};

#endif // HUMANDETECTIONMANAGE_H
